ObsPortal
=========

Application setup
-----------------

- Init a Python 3 virtualenv

    cd <directory containing this file>
    python3 -m venv .env

- Activate the virtualenv

    .env/bin/activate

- Now, we're assuming you are inside <directory containing this file> in the activated virtualenv!

- Upgrade packaging tools.

    pip3 install --upgrade pip setuptools wheel

- Install the project in editable mode

    pip3 install -e .

- (dev)
  Install development packages:
    pip install -e ".[develop]"

  Upgrade local packages:
    pip freeze > requirements.txt

    Edit requirements to :
        - remove -e git+git@gricad-gitlab.univ-grenoble-alpes.fr:OSUG/JMMC/jmmc-obsportal.git@...#egg=obsportal
        - change '==' by '>='
    pip install -r requirements.txt --upgrade


- With <application_settings_file> = production.ini OR development.ini

    cp <application_settings_file>-dist <application_settings_file>
    nano <application_settings_file>

    - And configure these parameters regarding to your deployment environment:

        - sqlalchemy.url : database connection string. See https://docs.sqlalchemy.org/en/latest/core/engines.html#postgresql

        - obsportal.paths.data : top-directory storing all data retrieved by the application
        - obsportal.paths.logs : top-directory storing all logs generated by the application
        - obsportal.paths.tmp : top-directory storing all temporary files generated by the application

        - obsportal.variant : blue/green variant using Docker/Kubernetes deployment

    - Or declare environment variables in place of hard-coded values in the INI file:

        - use UPCASED keys with spaces replaced by underscores (_)
        - always prefix an ENV key by 'OBSPORTAL_' only if the INI key doesn't begin by 'obsportal.' :
            - sqlalchemy.url => OBSPORTAL_SQLALCHEMY_URL
            - obsportal.variant => OBSPORTAL_VARIANT

        - export OBSPORTAL_VARIANT=blue
        - export OBSPORTAL_SQLALCHEMY_URL=postgresql://user:password@host/dbname?client_encoding=utf8


    - PLEASE DO NOT USE AND MODIFY *.ini-dist FILES, USE LOCAL *ini FILES INSTEAD
    - PLEASE DO NOT COMMIT LOCAL *.ini FILES, KEEP IT FOR YOUR OWN USAGE ONLY


Database setup
--------------

- If your postgresql role has admin powers:

    obsportal-cli database create --structure --debug --settings=<application_settings_file>

- Else, if your database already exists

    obsportal-cli database create-structure --debug --settings=<application_settings_file>


- Finally, load initial data:

    obsportal-cli database init --debug --settings=<application_settings_file>


Database (Alembic) migrations:
------------------------------

- Only once:

    Stamp the first revision (= DB structure at this time will be the top reference for Alembic versionning)
        DONE

- Each time, after initialization:

    After each modification on DB structure, generate a new Alembic revision.

        obsportal-cli database revision --settings=client.ini

    Review the SQL code generated by the revision. Add missing non-detected DB structure changes. Add DB content upgrade code.

- Apply the revision (already included in shell scripts):

        obsportal-cli database upgrade --settings=client.ini

- Test downgrade:
        obsportal-cli database downgrade 826d0b9dcafa --settings=client.ini
        obsportal-cli database upgrade --settings=client.ini


Database feeding
-------------------------

- Synchronize content from ESO/CHARA:

    obsportal-cli synchronize --debug --settings=<application_settings_file>
 
    obsportal-cli synchronize --services eso --limit 100

Launch the application
----------------------

pserve <application_settings_file> --reload

Run Aspro2 with local use
-------------------------

JAVA_OPTS=-DQueryRawObservations.local=true run_Aspro2-gui.sh


*******************
Build Docker image:
*******************

* For production:
    docker build -t "obsportal:latest" --build-arg IMAGE_TAG=YYYY.MM.DD[_bN] .
* For development:
    docker build -t "obsportal:latest" --build-arg IMAGE_TAG=YYYY.MM.DD_DEV --build-arg APPCONFFILE=development.ini --build-arg CLICONFFILE=client.ini .
    docker run -p 6543:6543 -v /data:/data:rw --rm --name d_obsportal "obsportal:latest"

* Initialize existing postgres DB:
    (see db/ scripts to initialize manually the postgres + pgsphere database)
    docker exec -it d_obsportal obsportal-cli database create-structure --skip_exist --debug --settings=client.ini
    docker exec -it d_obsportal obsportal-cli database init --debug --settings=client.ini

* Import Data (Ctrl-C to stop):
    docker exec -it d_obsportal-cli synchronize --debug --settings=client.ini

* Drop database content (BE CAREFUL):
    docker exec -it d_obsportal-cli database drop-structure --debug --settings=client.ini
    OR
    docker exec -it d_obsportal obsportal-cli database run-sql-script db/_clear_db.sql --settings=client.ini

* Use bash in container:
    docker exec -it d_obsportal bash



*******************
Publish release:
*******************

    TAG='2020_04_16'

    git checkout master
    git pull

    git flow release start $TAG
    git flow release finish $TAG

    git push
    git push --tags
    git checkout master
    git push

    git checkout develop


*******************
Launch unit tests
*******************

_ Install the project with 'testing' dependencies:
pip3 install -e ".[testing]"

- Copy testing.ini-dist to testing.ini and configure the parameters (see above)

- Ensure that database is running and feeded

- Run : obsportal-cli test


That's All, folks !

