"""First release (ObsPortal2020_04_16)

Revision ID: 5a7cecf8ad60
Revises:
Create Date: 2020-04-16 11:06:58.244786

"""
import sqlalchemy as sa
from alembic import op
from sqlalchemy_jsonfield import JSONField
from sqlalchemy_utc import UtcDateTime
from sqlalchemy_utils import UUIDType, ChoiceType
from sqlalchemy.sql import text

from obsportal.models.enums.observation import EnumObservationCategory
from obsportal.models.enums.program import EnumProgramType

# revision identifiers, used by Alembic.
revision = '5a7cecf8ad60'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.execute(text("CREATE SCHEMA IF NOT EXISTS obsportal"))

    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('interferometer',
                    sa.Column('id', sa.Unicode(), nullable=False),
                    sa.Column('name', sa.Unicode(), nullable=False),
                    sa.Column('latitude', sa.Float(), nullable=True),
                    sa.Column('longitude', sa.Float(), nullable=True),
                    sa.Column('altitude', sa.Float(), nullable=True),
                    sa.Column('headers_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.Column('observations_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.Column('exposures_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_interferometer')),
                    sa.UniqueConstraint('name', name=op.f('uq_interferometer_name')),
                    schema='obsportal'
                    )
    op.create_table('program',
                    sa.Column('id', UUIDType(), nullable=False),
                    sa.Column('identifier', sa.Unicode(), nullable=True),
                    sa.Column('alternative_identifier', sa.Unicode(), nullable=True),
                    sa.Column('period', sa.Integer(), nullable=True),
                    sa.Column('name', sa.Text(), nullable=True),
                    sa.Column('type', ChoiceType(EnumProgramType), nullable=False),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_program')),
                    schema='obsportal'
                    )
    op.create_table('target',
                    sa.Column('id', sa.Integer(), nullable=False),
                    sa.Column('name', sa.Unicode(), nullable=True),
                    sa.Column('ra', sa.Float(), nullable=True),
                    sa.Column('dec', sa.Float(), nullable=True),
                    sa.Column('observations_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.Column('exposures_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_target')),
                    sa.UniqueConstraint('name', 'ra', 'dec', name='uq_target_name_ra_dec'),
                    schema='obsportal'
                    )
    op.create_index(op.f('ix_obsportal_target_dec'), 'target', ['dec'], unique=False, schema='obsportal')
    op.create_index(op.f('ix_obsportal_target_name'), 'target', ['name'], unique=False, schema='obsportal')
    op.create_table('instrument',
                    sa.Column('id', sa.Unicode(), nullable=False),
                    sa.Column('interferometer_id', sa.Unicode(), nullable=False),
                    sa.Column('name', sa.Unicode(), nullable=False),
                    sa.Column('headers_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.Column('observations_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.Column('exposures_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.ForeignKeyConstraint(['interferometer_id'], ['obsportal.interferometer.id'],
                                            name=op.f('fk_instrument_interferometer_id_interferometer'),
                                            onupdate='CASCADE', ondelete='RESTRICT'),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_instrument')),
                    sa.UniqueConstraint('name', name=op.f('uq_instrument_name')),
                    schema='obsportal'
                    )
    op.create_index(op.f('ix_obsportal_instrument_interferometer_id'), 'instrument', ['interferometer_id'],
                    unique=False, schema='obsportal')
    op.create_table('program_run',
                    sa.Column('id', UUIDType(), nullable=False),
                    sa.Column('program_id', UUIDType(), nullable=True),
                    sa.Column('identifier', sa.Unicode(), nullable=True),
                    sa.Column('alternative_identifier', sa.Unicode(), nullable=True),
                    sa.ForeignKeyConstraint(['program_id'], ['obsportal.program.id'],
                                            name=op.f('fk_program_run_program_id_program'), onupdate='CASCADE',
                                            ondelete='RESTRICT'),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_program_run')),
                    schema='obsportal'
                    )
    op.create_table('header',
                    sa.Column('type', sa.String(length=5), nullable=True),
                    sa.Column('id', sa.Unicode(), nullable=False),
                    sa.Column('date_updated', UtcDateTime(timezone=True), nullable=True),
                    sa.Column('program_id', sa.Unicode(), nullable=True),
                    sa.Column('is_observation', sa.Boolean(), nullable=True),
                    sa.Column('dp_cat', sa.Unicode(), nullable=True),
                    sa.Column('dp_type', sa.Unicode(), nullable=True),
                    sa.Column('interferometer_id', sa.Unicode(), nullable=False),
                    sa.Column('instrument_id', sa.Unicode(), nullable=False),
                    sa.Column('mjd_start', sa.Float(), nullable=True),
                    sa.Column('file_path', sa.Unicode(), nullable=True),
                    sa.Column('file_hash', sa.String(), nullable=True),
                    sa.Column('extracted_content', JSONField(), nullable=True),
                    sa.Column('validation_log', JSONField(), nullable=True),
                    sa.ForeignKeyConstraint(['instrument_id'], ['obsportal.instrument.id'],
                                            name=op.f('fk_header_instrument_id_instrument'), onupdate='CASCADE',
                                            ondelete='RESTRICT'),
                    sa.ForeignKeyConstraint(['interferometer_id'], ['obsportal.interferometer.id'],
                                            name=op.f('fk_header_interferometer_id_interferometer'), onupdate='CASCADE',
                                            ondelete='RESTRICT'),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_header')),
                    schema='obsportal'
                    )
    op.create_index(op.f('ix_obsportal_header_instrument_id'), 'header', ['instrument_id'], unique=False,
                    schema='obsportal')
    op.create_index(op.f('ix_obsportal_header_interferometer_id'), 'header', ['interferometer_id'], unique=False,
                    schema='obsportal')
    op.create_index(op.f('ix_obsportal_header_mjd_start'), 'header', ['mjd_start'], unique=False, schema='obsportal')
    op.create_index(op.f('ix_obsportal_header_program_id'), 'header', ['program_id'], unique=False, schema='obsportal')
    op.create_table('instrument_mode',
                    sa.Column('id', sa.Integer(), nullable=False),
                    sa.Column('instrument_id', sa.Unicode(), nullable=False),
                    sa.Column('name', sa.Unicode(), nullable=False),
                    sa.Column('wavelength_min', sa.Float(), nullable=True),
                    sa.Column('wavelength_max', sa.Float(), nullable=True),
                    sa.Column('spectral_resolution', sa.Float(), nullable=True),
                    sa.Column('observations_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.Column('exposures_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.ForeignKeyConstraint(['instrument_id'], ['obsportal.instrument.id'],
                                            name=op.f('fk_instrument_mode_instrument_id_instrument'),
                                            onupdate='CASCADE', ondelete='RESTRICT'),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_instrument_mode')),
                    sa.UniqueConstraint('instrument_id', 'name', name='uq_instrument_mode_instrument_id_name'),
                    schema='obsportal'
                    )
    op.create_index(op.f('ix_obsportal_instrument_mode_instrument_id'), 'instrument_mode', ['instrument_id'],
                    unique=False, schema='obsportal')
    op.create_index(op.f('ix_obsportal_instrument_mode_name'), 'instrument_mode', ['name'], unique=False,
                    schema='obsportal')
    op.create_table('observation',
                    sa.Column('id', sa.Integer(), nullable=False),
                    sa.Column('category', ChoiceType(EnumObservationCategory), nullable=False),
                    sa.Column('program_id', sa.Unicode(), nullable=True),
                    sa.Column('interferometer_id', sa.Unicode(), nullable=False),
                    sa.Column('stations', sa.Unicode(), nullable=True),
                    sa.Column('pops', sa.Unicode(), nullable=True),
                    sa.Column('instrument_id', sa.Unicode(), nullable=False),
                    sa.Column('instrument_mode_id', sa.Integer(), nullable=True),
                    sa.Column('instrument_submode', sa.Unicode(), nullable=True),
                    sa.Column('target_id', sa.Integer(), nullable=True),
                    sa.Column('exposures_count', sa.Integer(), server_default=sa.text('0'), nullable=True),
                    sa.ForeignKeyConstraint(['instrument_id'], ['obsportal.instrument.id'],
                                            name=op.f('fk_observation_instrument_id_instrument'), onupdate='CASCADE',
                                            ondelete='RESTRICT'),
                    sa.ForeignKeyConstraint(['instrument_mode_id'], ['obsportal.instrument_mode.id'],
                                            name=op.f('fk_observation_instrument_mode_id_instrument_mode'),
                                            onupdate='CASCADE', ondelete='RESTRICT'),
                    sa.ForeignKeyConstraint(['interferometer_id'], ['obsportal.interferometer.id'],
                                            name=op.f('fk_observation_interferometer_id_interferometer'),
                                            onupdate='CASCADE', ondelete='RESTRICT'),
                    sa.ForeignKeyConstraint(['target_id'], ['obsportal.target.id'],
                                            name=op.f('fk_observation_target_id_target'), onupdate='CASCADE',
                                            ondelete='RESTRICT'),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_observation')),
                    schema='obsportal'
                    )
    op.create_index(op.f('ix_obsportal_observation_instrument_id'), 'observation', ['instrument_id'], unique=False,
                    schema='obsportal')
    op.create_index(op.f('ix_obsportal_observation_instrument_mode_id'), 'observation', ['instrument_mode_id'],
                    unique=False, schema='obsportal')
    op.create_index(op.f('ix_obsportal_observation_interferometer_id'), 'observation', ['interferometer_id'],
                    unique=False, schema='obsportal')
    op.create_index(op.f('ix_obsportal_observation_program_id'), 'observation', ['program_id'], unique=False,
                    schema='obsportal')
    op.create_index(op.f('ix_obsportal_observation_target_id'), 'observation', ['target_id'], unique=False,
                    schema='obsportal')
    op.create_table('exposure',
                    sa.Column('id', sa.Unicode(), nullable=False),
                    sa.Column('observation_id', sa.Integer(), nullable=False),
                    sa.Column('header_id', sa.Unicode(), nullable=False),
                    sa.Column('date_updated', UtcDateTime(timezone=True), nullable=True),
                    sa.Column('mjd_start', sa.Float(), nullable=True),
                    sa.Column('mjd_end', sa.Float(), nullable=True),
                    sa.Column('projected_baselines', JSONField(), nullable=True),
                    sa.Column('tau0', sa.Float(), nullable=True),
                    sa.Column('temperature', sa.Float(), nullable=True),
                    sa.Column('seeing', sa.Float(), nullable=True),
                    sa.Column('validation_level', sa.SmallInteger(), server_default=sa.text('0'), nullable=True),
                    sa.Column('invalid_fields', JSONField(), nullable=True),
                    sa.ForeignKeyConstraint(['header_id'], ['obsportal.header.id'],
                                            name=op.f('fk_exposure_header_id_header'), onupdate='CASCADE',
                                            ondelete='RESTRICT'),
                    sa.ForeignKeyConstraint(['observation_id'], ['obsportal.observation.id'],
                                            name=op.f('fk_exposure_observation_id_observation'), onupdate='CASCADE',
                                            ondelete='RESTRICT'),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_exposure')),
                    schema='obsportal'
                    )
    op.create_index(op.f('ix_obsportal_exposure_header_id'), 'exposure', ['header_id'], unique=False,
                    schema='obsportal')
    op.create_index(op.f('ix_obsportal_exposure_mjd_end'), 'exposure', ['mjd_end'], unique=False, schema='obsportal')
    op.create_index(op.f('ix_obsportal_exposure_mjd_start'), 'exposure', ['mjd_start'], unique=False,
                    schema='obsportal')
    op.create_index(op.f('ix_obsportal_exposure_observation_id'), 'exposure', ['observation_id'], unique=False,
                    schema='obsportal')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_obsportal_exposure_observation_id'), table_name='exposure', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_exposure_mjd_start'), table_name='exposure', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_exposure_mjd_end'), table_name='exposure', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_exposure_header_id'), table_name='exposure', schema='obsportal')
    op.drop_table('exposure', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_observation_target_id'), table_name='observation', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_observation_program_id'), table_name='observation', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_observation_interferometer_id'), table_name='observation', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_observation_instrument_mode_id'), table_name='observation', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_observation_instrument_id'), table_name='observation', schema='obsportal')
    op.drop_table('observation', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_instrument_mode_name'), table_name='instrument_mode', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_instrument_mode_instrument_id'), table_name='instrument_mode', schema='obsportal')
    op.drop_table('instrument_mode', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_header_program_id'), table_name='header', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_header_mjd_start'), table_name='header', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_header_interferometer_id'), table_name='header', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_header_instrument_id'), table_name='header', schema='obsportal')
    op.drop_table('header', schema='obsportal')
    op.drop_table('program_run', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_instrument_interferometer_id'), table_name='instrument', schema='obsportal')
    op.drop_table('instrument', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_target_name'), table_name='target', schema='obsportal')
    op.drop_index(op.f('ix_obsportal_target_dec'), table_name='target', schema='obsportal')
    op.drop_table('target', schema='obsportal')
    op.drop_table('program', schema='obsportal')
    op.drop_table('interferometer', schema='obsportal')

    op.execute(text("DROP SCHEMA IF EXISTS obsportal"))
    # ### end Alembic commands ###
